const express = require('express');
const app = express();
const http = require('http').createServer(app);
const cors = require('cors');
const md5 = require('md5');
const admZip = require('adm-zip');
const fs = require('fs');

const bodyParser = require('body-parser');
const consign = require('consign');
const path = require('path');
const serverConfig = require('../config/ServerConfig.js');
const jwt = require('jsonwebtoken');
// const compression = require('compression');

const watch = require('./app/model/watch');
const parametrizacoes = require('./app/model/parametrizacoes');
const logins = require('./app/model/loginModel');
const updaterController = require('./app/controller/UpdaterController');

app.use(bodyParser.json({ limit: '50mb' }));

app.use(cors());

consign({ cwd: 'app' });

consign({ cwd: path.resolve(__dirname, 'app'), loggin: false, verbose: false })
  .then('middleware')
  .then('logs')
  .then('model')
  .then('controller')
  .then('service')
  .then('routes')
  .into(app);

app.service.SocketService.start(http);

app.post('/add', ((req, res) => {
  res.send(`Text: ${JSON.stringify(req.body)}`);
}));

app.post('/login', ((req, res) => {
  logins.getLogin({ PASSWORD: md5(req.body.password), EMAIL: req.body.email }).then((login) => {
    if (login.rows.length === 0) {
      res.send('Email ou senha inválidos');
    } else {
      res.send({
        CDOPERADOR: login.rows[0][0],
        NMOPERADOR: login.rows[0][1],
        IDOPERACESSO: login.rows[0][2]
      });
    }
    // console.log(req.body, login);
  }).catch(((error) => {
    res.send('Erro');
  }));
}));

/** Rota para buscar os dados do component
 * json da tela: data/monitoring?v=1635352485854
 * Json de configurações: /data/connect?obj=monitoring
 */
app.get('/data/*', ((req, res) => {
  /** Tipo de conexões dos monitoramentos */
  if (req.path.split('/')[2] === 'connect') {
    let conexao = [];
    let ave = [];

    /** Rota simples */
    if (req.query.obj !== undefined && req.query.obj !== 'monitoring') {
      watch.getAll({ NOME: req.query.obj })
        .then((a => {
          res.send(a.rows[0][3]);
        })).catch((() => {
          res.send('Component não registrado.');
        }));
    }
    /** Conexões da Home, lista todas e retorna */
    else {
      watch.getConexao({ NAVEGACAO: 'component' })
        .then((a => {
          conexao = a.rows[0][0];
          ave = JSON.parse(conexao).conexao;

          /** As conexões filhos */
          watch.getConexao({ NAVEGACAO: 'monitoring' })
            .then((a => {
              a.rows.forEach((element) => {
                const conect = JSON.parse(element[0]).conexao[1];
                ave.push(conect);
              });
              const rest = JSON.parse(conexao);
              rest.conexao = ave;
              res.send(rest);

            })).catch((() => {
              res.send('Component não registrado.');
            }));

        })).catch((() => {
          res.send('Component não registrado.');
        }));
      }
  } else if (req.path.split('/')[2] === 'parametrizacoes') {
    parametrizacoes.get({ MIN_ROW: 1, MAX_ROW: 10 })
      .then((a) => {
        const parans = [];
        a.rows.forEach((element) => {
          parans.push({
            id: element[0],
            nome: element[1],
            component: element[2],
            conexao: element[3],
            status: element[4],
            navegacao: element[5],
            ordem: element[6],
            item: element[7],
          });
        });
        res.send(parans);
      }).catch((error) => {
        res.send('Erro ao buscar os components.');
      });
  } else {
    watch.getAll({ NOME: req.path.split('/')[2] })
      .then((a) => {
        /** Se for o monitoring, busque as informações de Colunas */
        if (a.rows[0][1] === 'monitoring') {
          const componente = JSON.parse(a.rows[0][2]).children[0].children[0].columns;
          watch.getNav()
            .then((colunas) => {
              if (colunas.rows.length > 0) {
                colunas.rows.forEach((col) => {
                  componente.push(JSON.parse(col[0]));
                });
              }
              const novoComponent = JSON.parse(a.rows[0][2]);
              novoComponent.children[0].children[0].columns = componente;
              res.send(novoComponent);
            });
        } else {
          res.send(a.rows[0][2]);
        }
      }).catch((error) => {
        res.send('Component não registrado.');
      });
  }
}));

app.get('/versoes', (req, res) => {
  const novoComponent = JSON.stringify(
    {
      "data":[{
        "id": "9999",
        "cd": "9999",
        "nome": "Filial A",
        "app": "-",
        "sub_app": "-",
        "v": "-",
        "vd": ""
      },
      {
        "id": "9999.1",
        "cd": "9999",
        "nome": "Filial A",
        "PARENT_PROJECT": "9999",
        "devOpsId": "2315",
        "app": "Odhen EXP",
        "sub_app": "-",
        "v": "3.5.1",
        "vd": ""
      },
      {
        "id": "9999.2",
        "cd": "9999",
        "nome": "Filial A",
        "PARENT_PROJECT": "9999",
        "devOpsId": "2310",
        "app": "Odhen Updater",
        "sub_app": "-",
        "v": "2.8.6",
        "vd": ""
      },
      {
        "id": "9999.3.1",
        "cd": "9999",
        "nome": "Filial A",
        "PARENT_PROJECT": "9999",
        "devOpsId": "9311",
        "app": "Monitoramento",
        "sub_app": "Vendas",
        "v": "0.0.1",
        "vd": "0.0.2"
      },
      {
        "id": "9999.3.2",
        "cd": "9999",
        "nome": "Filial A",
        "PARENT_PROJECT": "9999",
        "devOpsId": "9311",
        "app": "Monitoramento",
        "sub_app": "Delivery",
        "v": "0.0.2",
        "vd": "0.0.3"
      },
      {
        "id": "9999.3.3",
        "cd": "9999",
        "nome": "Filial A",
        "PARENT_PROJECT": "9999",
        "devOpsId": "9311",
        "app": "Monitoramento",
        "sub_app": "Sat",
        "v": "0.0.2",
        "vd": ""
      }]
      ,"pagination":{"page":1,"limit":10,"total":10}}
  );
  res.send(novoComponent);
});

app.put('/data/connect/portal/*', ((req, res) => {
  watch.upElemnet({
    STATUS: req.body.status,
    ORDEM: req.body.ordem,
    COMPONENT: req.body.component,
    ITEM: req.body.item,
    CONEXAO: req.body.conexao,
    ID: req.body.id
  }).then((a) => {
    a.connection.commit();
    res.send('Sucesso!');
  }).catch((error) => {
    res.send(error);
  });
}));

/**
 * @author Daniel Souza <daniel.souza@teknisa.com>
 * Rota para fazer download dos softwares Teknisa
 * @exemple http://localhost:3071/download?aplicacao=daniel&versao=v0.0.1
*/
app.get('/download', ((req, res) => {
  const aplicacao = req.query.aplicacao.toUpperCase();
  const versao = req.query.versao.toUpperCase();

  const dirApp = 'repositories/aplicacoes/';
  /** Repositório */
  let dir = `${dirApp + aplicacao}/${versao}`;
  dir = path.join(`${__dirname}`, dir);

  /** Se não existir retorna */
  if (!fs.existsSync(dir)) {
    return res.send('Aplicação não disponível!');
  }

  /** Repositorio zipadao */
  const name = `${dirApp + aplicacao}-${versao}.zip`;
  const dirZip = path.join(`${__dirname}`, name);
  const zip = new admZip();
  let files = [];
  files = fs.readdirSync(dir);

  files.forEach((file) => {
    if (fs.statSync(`${dir}/${file}`).isDirectory()) {
      zip.addLocalFolder(path.resolve(dir, file), file);
    } else {
      zip.addLocalFile(`${dir}/${file}`);
    }
  });

  zip.writeZip(dirZip);
  const buff = zip.toBuffer();
  /** arquivo nome DANIEL-V0.0.1.zip */
  fs.unlinkSync(dirZip);

  /** Permite o download do zip */
  return res.send(buff);
}));

/**
 * @version 1
 * @author Daniel Souza <daniel.souza@teknisa.com>
 * @parans serviceId=odhen-updater
 * @return zip.toBuffer()
 */
app.get('/versions', (req, res) => {
  // console.log('versions:', req);
  let serviceId = '';

  if (typeof req.query.serviceId === 'string') {
    serviceId = req.query.serviceId;
  }

  if (!serviceId) {
    return res.status(400).json({
      message: 'Service Name cannot be null.',
    });
  }

  const aplicacao = req.query.serviceId.toUpperCase();
  const dirApp = 'repositories/aplicacoes/';
  let dir = `${dirApp + aplicacao}/`;
  dir = path.join(__dirname, dir);

  if (!dir) {
    return res.status(400).json({ error: 'Service folder not found' });
  }

  const zip = new admZip();
  const files = fs.readdirSync(dir);

  files.forEach((file) => {
    if (fs.statSync(dir + file).isDirectory()) {
      zip.addLocalFolder(path.resolve(dir, file), file);
    } else {
      zip.addLocalFile(`${dir}/${file}`);
    }
  });

  const name = `${dirApp + aplicacao}}.zip`;
  const dirZip = path.join(`${__dirname}`, name);
  zip.writeZip(dirZip);
  const buff = zip.toBuffer();
  /** arquivo nome DANIEL-V0.0.1.zip */
  fs.unlinkSync(dirZip);

  return res.send(buff);
});

/**
 * @author Daniel Souza <daniel.souza@teknisa.com>
 * @parans serviceId=odhen-updater
 * @return {"version":3006000}
 */
app.get('/latest', (req, res) => {
  let serviceId = '';
  const aplicacao = req.query.serviceId.toUpperCase();
  // console.log('latest:', req);

  if (typeof req.query.serviceId === 'string') {
    serviceId = req.query.serviceId;
  }

  if (!serviceId) {
    return res.status(400).send({ error: true, message: '' });
  }

  /** Repositório */
  const dirApp = 'repositories/aplicacoes/';
  let dir = `${dirApp + aplicacao}`;

  dir = path.join(__dirname, dir);
  const productInfo = JSON.parse(fs.readFileSync(`${dir}/productInfo.json`));

  if (!productInfo) {
    return null;
  }

  let { version } = productInfo.product;

  if (!version) {
    return res.status(400).send({ error: true, message: 'Versão não encontrada.' });
  }

  version = formatVersion(version);

  return res.status(200).json({ version });
});

/**
 * @version 1
 * @param {*} version
 * @returns 3006000 (3.6.0)
 */
function formatVersion(version) {
  if (!version) {
    return null;
  }

  const versionParts = version.split('.').map((v) => {
    return v.padStart(3, '0');
  });

  return parseInt(versionParts.join(''), 10);
}

/**
 * @version 1
 * @author Daniel Souza <daniel.souza@teknisa.com>
 * @param {*} version
 * @returns 3006000 (3.6.0)
 */
 app.get('/updater/pendentes', (req, res) => {
  updaterController.getVersaoDisponivel(req.query).then((services) => {
    return res.status(200).json({ services });
  }).catch((err) => {
    console.log('/updater/pendentes - Erro: ', err);
    return res.status(400).json({ error: 'Erro ao buscar uma versão disponível' });
  });
});

/**
 * Atualiza a tabela informando se o sistema foi atualizado ou se deu erro
 * @version 1
 * @author Daniel Souza <daniel.souza@teknisa.com>
 * @param {*} {"IDUPDATERVERSAO": 26, "UPVERSTATUS": 0}
 * @returns
 */
app.put('/updater/*', (req, res) => {
  updaterController.updateVersaoDisponivel(req.body).then((services) => {
    if (services.rowsAffected > 0) {
      return res.status(200).json({ services });
    }
    return res.status(400).json({ error: 'Dados incorretos' });
  }).catch((err) => {
    return res.status(400).json({ error: 'Erro ao buscar uma versão disponível' });
  });

});


//
let customDbname = !!process.argv[2] ? process.argv[2] : '';
let customPort = !!process.argv[3] ? process.argv[3] : '';

if (customDbname.indexOf('USR_ORG_') != -1) {
  customPort = parseInt(customDbname.slice(8, customDbname.length));
}

if (!(customDbname.indexOf('USR_ORG_') != -1) && !customPort) {
  customPort = 7777;
}

if (customDbname.indexOf('ODHEN') != -1) {
  customDbname = '';
}

http.listen(customPort, (_) => {
  console.log(`Running on ${customPort} port at ${customDbname}!`);
});

/** Instancia o front */
app.use(express.static(path.resolve(__dirname, '../frontend/dist')));
/** Faz com que o node utilize as rotas do front, mas as rotas não podem ter o mesmo path */
app.use('*', express.static(path.resolve(__dirname, '../frontend/dist')));