import { SocketEvent } from '@/socket/socket-event';
import {
  Config,
  IDictionary,
  IEventParam,
  Loader,
} from '@zeedhi/core';
import { TekGrid } from '@zeedhi/teknisa-components-common';
import { io, Socket } from 'socket.io-client';
import { AlertService } from '@zeedhi/common';

export class MonitoringController {
  public socket?: Socket;

  /** Style Status do Sat */
  public getConditionalStyle({ row }: IDictionary<any>) {
    if (row.sat === 'true') {
      return 'green';
    }
    return 'red';
  }

  /** Menssagem Status do Sat */
  public getConditionalMessageSat({ row }: IDictionary<any>) {
    if (
      (row.sat === '') && (row.socketId !== '')
    ) {
      return 'Erro ao tentar uma comunicação com o SAT.';
    }
    return row.mensagem;
  }

  /** Menssagem Status do Vendas */
  public getConditionalMessageSales({ row }: IDictionary<any>) {
    if (
      (row.vendas === false) && (row.socketId !== '')
    ) {
      return 'Você tem vendas que não foram sincronizadas.';
    }
    return row.mensagem;
  }

  /** Menssagem Status do Delivery */
  public getConditionalMessageDelivery({ row }: IDictionary<any>) {
    if (
      (row.delivery === false) && (row.socketId !== '')
    ) {
      return 'Você tem vendas que não foram sincronizadas com a loja.';
    }
    return row.mensagem;
  }

  /** Menssagem Status do Parameters */
  public getConditionalMessageParameters({ row }: IDictionary<any>) {
    if (
      (row.parametro === false) && (row.socketId !== '')
    ) {
      return 'Verifique os parâmetros da loja.';
    }
    return row.mensagem;
  }

  /** Menssagem Status do NF Sefaz */
  public getConditionalMessageSefaz({ row }: IDictionary<any>) {
    if (
      (row.parametro === false) && (row.socketId !== '')
    ) {
      return 'Webservice da Sefaz encontra-se com disponibilidade.';
    }
    return row.mensagem;
  }

  /** Style Status do Socket */
  public getConditionalStyleWebsocket({ row }: IDictionary<any>) {
    if (row.situacao === 'Ativo') {
      return 'green';
    }
    return 'red';
  }

  /** Style Status do Vendas */
  public getConditionalStyleSales({ row }: IDictionary<any>) {
    if (row.vendas === true) {
      return 'green';
    }
    return 'red';
  }

  /** Style Status do Delivery */
  public getConditionalStyleDelivery({ row }: IDictionary<any>) {
    if (row.delivery === true) {
      return 'green';
    }
    return 'red';
  }

  /** Style Status do Parameters */
  public getConditionalStyleParameters({ row }: IDictionary<any>) {
    if ((row.parametro === true) && (row.socketId !== '')) {
      return 'green';
    }
    if ((row.parametro === false) && (row.socketId !== '')) {
      return 'red';
    }
    if ((row.parametro === undefined) && (row.situacao !== 'Inativa')) {
      return 'red';
    }
    return 'red';
  }

  /** Style Status do NF Sefaz */
  public getConditionalStyleSefaz({ row }: IDictionary<any>) {
    // console.warn(row.id, row);
    if ((row.parametro === true) && (row.socketId !== '')) {
      return 'green';
    }
    if ((row.parametro === undefined) && (row.situacao !== 'Inativa')) {
      return 'green';
    }
    return 'red';
  }

  public startModal() {
    Loader.getInstance('SatController').createFullscreenModal();
    Loader.getInstance('SalesController').createFullscreenModal();
    Loader.getInstance('DeliveryController').createFullscreenModal();
    Loader.getInstance('ParametersController').createFullscreenModal();
    Loader.getInstance('SefazController').createFullscreenModal();
  }

  /**
   * Método executado quando o grid é criado
   */
  public gridMounted({ component }: IEventParam<TekGrid>) {
    // inicializa o socket
    // console.warn(process.env.NODE_ENV === 'development');
    // console.warn(Config.endPoint);
    this.socket = io(Config.endPoint);

    if (this.socket.connected === false) {
      console.warn(this.socket);
    }
    this.socket.on('disconnect', () => {
      AlertService.show({
        name: 'buttons-alert',
        text: 'A página será recarregada para restabelecer a conexão.',
        color: 'error',
        right: true,
        top: true,
        value: false,
        buttonsSlot: [
          {
            name: 'undo-button',
            component: 'ZdButton',
            icon: true,
            iconName: 'close',
            color: 'white',
            events: {
              click: () => { AlertService.hide(); },
            },
          },
        ],
      });

      setTimeout(() => {
        document.location.reload();
      }, 5000);
    });

    // registra um listener para o evento de connection
    this.socket.off(SocketEvent.CONNECTION)
      .emit('login', {
        email: 'francismar.araujo@teknisa.com',
        password: 'nineninenine',
        slave: false,
      });

    this.socket.off(SocketEvent.CONNECTION)
      .on('confirmLogin', async (branc: IDictionary) => {
        console.warn(branc);
        await component.datasource.updateData(branc.message);
      });

    /** SAT */
    this.socket.off(SocketEvent.CONNECTION)
      .emit('sat', {
        slave: false,
      });

    this.socket.off(SocketEvent.CONNECTION)
      .on('sat', async (branc: IDictionary) => {
        if (branc.message.vendas.length > 0) {
          branc.message.data.forEach((element: any) => {
            component.datasource.get().then((r: IDictionary) => {
              const coponetAtual = r.filter((v: IDictionary) => {
                if (element.id === v.CDFILIAL) {
                  v.sat = element.STATUS;
                  return v;
                }
                return false;
              });
              component.datasource.put(coponetAtual[0]);
            });
          });
        }
        await component.datasource.get();
      });

    /** Vendas */
    this.socket.off(SocketEvent.CONNECTION)
      .emit('saleStatus', {
        slave: false,
      });

    this.socket.off(SocketEvent.CONNECTION)
      .on('saleStatus', async (branc: IDictionary) => {
        if (branc.message.vendas.length > 0) {
          branc.message.vendas.forEach((element: any) => {
            component.datasource.get().then((r: IDictionary) => {
              const coponetAtual = r.filter((v: IDictionary) => {
                if (element.id === v.codigo) {
                  return v;
                }
                return false;
              });
              coponetAtual[0].vendas = false;
              component.datasource.put(coponetAtual[0]);
            });
          });
        }
        await component.datasource.get();
      });

    /** Vendas Delivery */
    this.socket.off(SocketEvent.CONNECTION)
      .emit('saleDeliveryStatus', {
        slave: false,
      });

    this.socket.off(SocketEvent.CONNECTION)
      .on('saleDeliveryStatus', async (branc: IDictionary) => {
        if (branc.message.delivery.length > 0) {
          branc.message.delivery.forEach((element: any) => {
            component.datasource.get().then((r: IDictionary) => {
              const coponetAtual = r.filter((v: IDictionary) => {
                if (element.id === v.id) {
                  return v.delivery;
                }
                return false;
              });
              if (coponetAtual.length > 0) {
                coponetAtual[0].delivery = false;
                component.datasource.put(coponetAtual[0]);
              }
            });
          });
        }
        await component.datasource.get();
      });

    /** PARAMETERS */
    this.socket.off(SocketEvent.CONNECTION)
      .emit('parameters', {
        slave: false,
      });

    this.socket.off(SocketEvent.CONNECTION)
      .on('parameters', async (branc: IDictionary) => {
        if (branc.message.parameters.length > 0) {
          branc.message.parameters[0].forEach((element: any) => {
            component.datasource.get().then((r: IDictionary) => {
              const coponetAtual = r.filter((v: IDictionary) => {
                if (element.CDFILIAL === v.id) {
                  return v;
                }
                return false;
              });
              coponetAtual[0].parametro = false;
              component.datasource.put(coponetAtual[0]);
            });
          });
        }
        await component.datasource.get();
      });
  }

  /**
   * Método que recebe a filial para a navegação
   */
  public ids(obj:any) {
    console.warn(obj);
  }

  public push(ids:any) {
    console.warn(ids);
    // const routFilial = `monitoring/${ids.row.id}/${ids.column.name} `;
    // Router.push(routFilial);
  }

  /**
   * Método executado logo antes do grid ser destruído
   */
  public gridBeforeDestroy() {

  }
}
